

		List	p=16f628			;tell assembler what chip is being used
		#include	"P16f628.inc"		;include the defaults for the chip
		__config	0x3D18			;sets configuration setting (oscillator type etc)

;RB1 is used to drive the relay
;RB3 is the switch fault LED
;RA0 is the microswitch input
;RA1, RA2 and RA3 is the dip switch selection for the Power_On time
;RA4, RA5 and RA5 is the dip switch selection for the off time when the microswitch input is activated

			org	0x0000				;sets the origin, 0x0000 for the 16f628
								;program starts running here
				cblock	0x20			;start of general purpose registers
				d1				;used in BaseTime 30 sec delay routine
				d2				;used in BaseTime 30 sec delay routine
				d3				;used in BaseTime 30 sec delay routine
				endc


				MOVLW	b'00000000'		;Reset all outputs - PORTB
				MOVWF	PORTB
				MOVLW	b'00000111'		;	
				MOVWF	CMCON			;turn comparators off to make it like a 16f84

				bsf	STATUS,RP0		;select bank 1
				MOVLW	b'00000000'		;set PortB all outputs
				MOVWF	TRISB			;
				MOVLW	b'11111111'		;set PortA all inputs
				MOVWF	TRISA			;
				bcf	STATUS,RP0		;select bank 0
		
				MOVLW	b'00000000'		;Reset all outputs - PORTB
				MOVWF	PORTB			;
				CALL	PowerOn_Time		;Check Power on delay time	- delay before switching relay on
				CALL	Out_On			;Set the output on ie relay on	
MAIN_1				CALL 	SwitchScan		;Scan switch input
				CALL	Out_Off			;Set output
				CALL	Time			;Set duration time of output (relay) off
				CALL	Out_On			;Switch relay back on after time delay
				GOTO	MAIN_1

;This routine checks the microswitch input

SwitchScan			BCF	PORTB,3			;Clear Fault LED
				btfss	PORTA,0			;check if switch on
				goto	SwitchScan		;if switch not on, loop and continue checking!
				GOTO	Release
				RETLW	00	
	
		
Release				MOVF	PORTA,w			;Move port a to w register
				ANDLW	b'00000001'		;mask off bit 0	
				MOVWF	27h			;Move result to reg 27
				MOVLW	00			;					
				XORWF	27h,0			;Compare file PortA bit 0 with 0 			
				BTFSS	STATUS,Z		;Look at zero flag   Z=Set=Same	
				GOTO	SwitchFault		;If not released fault output		
				BCF	PORTB,3			;Clear fault led
				RETLW	00			;Return 

SwitchFault			BSF	PORTB,3			;Switch fault PORTB bit3
				GOTO	Release			;which is the Fault LED
				RETURN	

		
PowerOn_Time			MOVF	PORTA,w			;read input from port A
				ANDLW	b'00001110'		;mask off bits - read only A1 to A3
				MOVWF	25h			;store A1 to A3 in file register 25 
  				MOVLW	b'00000000'		;
				XORWF	25h,0			;compare porta A1 to A3 with 000 - 30sec
				BTFSC	STATUS,Z		;If same Z bit=1 
				GOTO	Time1			;                                                                           		
				MOVLW	b'00000100'		;
				XORWF	25h,0			;compare porta A1 to A3 with 010 - 60sec
				BTFSC	STATUS,Z		;If same Z bit=1
				GOTO	Time2			;
				MOVLW	b'00000110'		;
				XORWF	25h,0			;compare porta A1 to A3 with 011 - 90sec
				BTFSC	STATUS,Z		;If same Z bit=1
				GOTO	Time3	
				MOVLW	b'00001000'		;compare porta A1 to A3 with 001 - 120sec
				XORWF	25h,0			;
				BTFSC	STATUS,Z		;if same Z bit =1
				GOTO	Time4			;
				MOVLW	b'00001010'		;compare porta A1 to A3 with 101 - 150sec
				XORWF	25h,0			;
				BTFSC	STATUS,Z		;if same Z bit =1
				GOTO	Time5			;
				MOVLW	b'00001100'		;compare porta A1 to A3 with 110 - 180sec
				XORWF	25h,0			;
				BTFSC	STATUS,Z		;if same Z bit =1
				GOTO	Time6			;

Out_On				bsf	PORTB,1			;Turn PORTB bit 1 on
				retlw	00

Out_Off				bcf	PORTB,1			;Turn PORTB bit 1 off
				retlw	00

Time				MOVF	PORTA,w			;read input from port A
				ANDLW	b'0111000'		;mask off bits - read only A4 to A6
				MOVWF	28h				;store A4 to A6 in file register 28                                                                                     		
				MOVLW	b'00000000'		;
				XORWF	28h,0			;compare porta A4 to A6 with 000 - 30 sec
				BTFSC	STATUS,Z		;If same Z bit=1 
				GOTO	Time1			;                                                                           		
				MOVLW	b'00100000'		;
				XORWF	28h,0			;compare porta A4 to A6 with 010 -60 sec
				BTFSC	STATUS,Z		;If same Z bit=1
				GOTO	Time2			;
				MOVLW	b'00110000'		;
				XORWF	28h,0			;compare porta A4 to A6 with 011 - 90 sec
				BTFSC	STATUS,Z		;If same Z bit=1
				GOTO	Time3	
				MOVLW	b'01000000'		;
				XORWF	28h,0			;compare porta A4 to A6 with 100 - 120 sec
				BTFSC	STATUS,Z		;If same Z bit=1 
				GOTO	Time4			;                                                                           		
				MOVLW	b'01010000'		;
				XORWF	28h,0			;compare porta A4 to A6 with 101 -150 sec
				BTFSC	STATUS,Z		;If same Z bit=1
				GOTO	Time5			;
				MOVLW	b'01100000'		;
				XORWF	28h,0			;compare porta A4 to A6 with 110 - 180 sec
				BTFSC	STATUS,Z		;If same Z bit=1
				GOTO	Time6
				
			
Time1				MOVLW	1h			;put value of 1 in time register (29)
				MOVWF	29h			;
				CALL	BaseTime		;call timer routine
				RETLW	00			;return
Time2				MOVLW	2h			;put value of 2 in time register (29)
				MOVWF	29h			;
				CALL	BaseTime		;call timer routine
				RETLW	00			;return
Time3				MOVLW	3h			;put value of 3 in time register (29)
				MOVWF	29h			;
				CALL	BaseTime		;call timer routine
				RETLW	00			;return
Time4				MOVLW	4h			;put value of 4 in time register (29)
				MOVWF	29h			;
				CALL	BaseTime		;call timer routine
				RETLW	00			;return
Time5				MOVLW	5h			;put value of 5 in time register (29)
				MOVWF	29h			;
				CALL	BaseTime		;call timer routine
				RETLW	00			;return
Time6				MOVLW	6h			;put value of 6 in time register (29)
				MOVWF	29h			;
				CALL	BaseTime		;call timer routine
				RETLW	00			;return

BaseTime							; Delay = 30 seconds multiplied by value in reg 29h
								; Clock frequency = 4 MHz

								; Actual delay = 30 seconds = 30000000 cycles
								; Error = 0 %

Tdelay
										;29999996 cycles
				movlw	0x11
				movwf	d1
				movlw	0x66
				movwf	d2
				movlw	0x42
				movwf	d3

Delay_0
				decfsz	d1, f
				goto	$+2
				decfsz	d2, f
				goto	$+2
				decfsz	d3, f
				goto	Delay_0

								;4 cycles 
				goto	$+1
				goto	$+1
			
				decfsz	29h,f			;reg 29 holds multiplier no.
				goto	Tdelay
				return	
				end

;Notes: 
;Register 29 holds the time multiplier.The base time is 30 secs.
;If reg 29 has a value of 2, then time = 2 x 30 secs
;Time routines generated by "delay code generator"
		
